@@ -21,7 +21,6 @@ dplyr_col_modify.mdl_df <- function(data, cols) {
 #' @export
 dplyr_reconstruct.mdl_df <- function(data, template) {
   res <- NextMethod()
-  build_mable(data,
-              key = !!key_vars(template), 
-              model = !!intersect(mable_vars(template), colnames(res)))
+  mbl_vars <- names(which(vapply(data, inherits, logical(1L), "lst_mdl")))
+  build_mable(data, key = !!key_vars(template), model = !!mbl_vars)
 }
