@@ -47,4 +47,12 @@ generics::generate
 
 #' @importFrom generics refit
 #' @export
-generics::refit
\ No newline at end of file
+generics::refit
+
+#' @importFrom distributional hilo
+#' @export
+distributional::hilo
+
+#' @importFrom distributional guide_level
+#' @export
+distributional::guide_level
\ No newline at end of file
